#!/usr/bin/python
from oscilib import (
    GREEN,
    AMBER,
    RED,
    WWW_ROOT,
    get_builds_from_jenkins,
    last_build_worked,
    write_spec_status_file,
    render_html,
)

REPORT_DAYS = 7
STATUS_FILE = WWW_ROOT + 'unit-test-status.html'
SUMMARY_TEMPLATE = 'unit-test-job-summary.tmpl'


def extract_build_info(build):
    build_info = build.name.split()
    if len(build_info) != 3:
        return None
    bundle = build_info[2]
    state = build.get_status()
    url = build.get_result_url()
    build_timestamp = build.get_timestamp()
    build_dict = {
        'bundle': bundle,
        'state': state,
        'buildtime': build_timestamp,
        'url': url,
    }
    return build_dict


def get_spec_dict(builds):
    spec_state = {}
    for build in builds:
        build_info = extract_build_info(build)
        if not build_info:
            continue
        bundle = build_info['bundle']
        if spec_state.get(bundle):
            spec_state[bundle].append(build)
        else:
            spec_state[bundle] = [build]
    return spec_state


def get_cell_data(spec_dict):
    rows = []
    fname = '/home/ubuntu/jenkins/charms.txt'
    with open(fname) as f:
        charms = f.readlines()
    for charm in charms:
        charm = charm.rstrip()
        row = [{'color': '', 'link': '', 'msg': charm}]
        if charm in spec_dict.keys():
            builds = spec_dict[charm]
            ubuntu = 'na'
            spec_status_file = write_spec_status_file(
                charm, ubuntu, builds, extract_build_info)
            if last_build_worked(builds):
                # Spec has run and worked
                row.append({
                    'colour': GREEN, 'link': spec_status_file,
                    'msg': 'Detail'
                })
            else:
                # Spec has run and failed
                row.append({
                    'colour': RED, 'link': spec_status_file,
                    'msg': 'Detail'
                })
        else:
            # Spec has not run
            row.append({'colour': AMBER, 'link': '', 'msg': ''})
        rows.append(row)
    return rows


if __name__ == '__main__':
    builds = get_builds_from_jenkins(['charm_unit_test'], REPORT_DAYS)
    spec_dict = get_spec_dict(builds)
    render_html(
        spec_dict,
        SUMMARY_TEMPLATE,
        get_cell_data,
        REPORT_DAYS,
        STATUS_FILE
    )
