#!/usr/bin/python
import yaml
from oscilib import (
    FULL_COMBOS,
    GREEN,
    GREY,
    AMBER,
    RED,
    WWW_ROOT,
    get_builds_from_jenkins,
    last_build_worked,
    write_spec_status_file,
    render_html,
)

JOB_FILE = '/home/ubuntu/jenkins/mojo-runner-job-map.yaml'
REPORT_DAYS = 7
STATUS_FILE = WWW_ROOT + 'mojo-status.html'
SUMMARY_TEMPLATE = 'mojo-job-summary.tmpl'


def extract_build_info(build):
    build_info = build.name.split()
    if len(build_info) < 5 or build_info[3] != '@':
        return None
    mojo_spec = build_info[2].strip()
    ubuntu_release = build_info[4]
    state = build.get_status()
    url = build.get_result_url()
    build_timestamp = build.get_timestamp()
    build_dict = {
        'spec': mojo_spec,
        'ubuntu_release': ubuntu_release,
        'state': state,
        'buildtime': build_timestamp,
        'url': url,
    }
    return build_dict


def get_spec_dict(builds):
    spec_state = {}
    for build in builds:
        build_info = extract_build_info(build)
        if not build_info:
            continue
        spec = build_info['spec']
        ubuntu_release = build_info['ubuntu_release']
        if not spec_state.get(spec):
            spec_state[spec] = {}
        if spec_state[spec].get(ubuntu_release):
            spec_state[spec][ubuntu_release].append(build)
        else:
            spec_state[spec][ubuntu_release] = [build]
    return spec_state


def get_cell_data(spec_dict):
    stream = file(JOB_FILE, 'r')
    PUBLISH_JOBS = yaml.load(stream)
    rows = []
    # cell: {'color': RED, 'link': 'http://', 'msg': 'hi'}
    for base_spec in PUBLISH_JOBS.keys():
        row = [{'color': '', 'link': '', 'msg': base_spec}]
        for combo in FULL_COMBOS:
            ubuntu, openstack = combo.split('/')
            spec = '{}/{}'.format(base_spec, openstack)
            if combo not in PUBLISH_JOBS[base_spec]:
                # If the ubuntu/openstack release combo is not listed as valid
                # for this sepc then grey out the cell and skip to the next
                row.append({'colour': GREY, 'link': '', 'msg': ''})
                continue
            if spec_dict.get(spec):
                if spec_dict[spec].get(ubuntu):
                    builds = spec_dict[spec][ubuntu]
                    spec_status_file = write_spec_status_file(
                        spec, ubuntu, builds, extract_build_info)
                    if last_build_worked(builds):
                        # Spec has run and worked
                        row.append({
                            'colour': GREEN, 'link': spec_status_file,
                            'msg': 'Detail'
                        })
                    else:
                        # Spec has run and failed
                        row.append({
                            'colour': RED, 'link': spec_status_file,
                            'msg': 'Detail'
                        })
                else:
                    # Spec has not run
                    row.append({'colour': AMBER, 'link': '', 'msg': ''})
            else:
                # Spec has not run
                row.append({'colour': AMBER, 'link': '', 'msg': ''})
        rows.append(row)
    return rows


if __name__ == '__main__':
    builds = get_builds_from_jenkins(
        ['mojo_runner', 'mojo_runner_baremetal'], REPORT_DAYS)
    spec_dict = get_spec_dict(builds)
    render_html(
        spec_dict,
        SUMMARY_TEMPLATE,
        get_cell_data,
        REPORT_DAYS,
        STATUS_FILE
    )
