from jenkinsapi.jenkins import Jenkins
from time import gmtime, strftime
from datetime import date, timedelta
from jinja2 import Environment, FileSystemLoader

JENKINS_URL = 'http://10.245.162.77:8080'
WWW_ROOT = '/var/www/html/'
TEMPLATES = '/home/ubuntu/jenkins/templates'
FULL_COMBOS = ['precise/icehouse', 'trusty/icehouse', 'trusty/juno',
               'trusty/kilo', 'trusty/liberty', 'vivid/kilo', 'wily/liberty']
GREEN = '#00FF00'
GREY = '#808080'
AMBER = '#FFD700'
RED = '#FF0000'


def get_server_instance():
    server = Jenkins(JENKINS_URL)
    return server


def get_builds_from_jenkins(job_names, report_days):
    jobs = []
    server = get_server_instance()
    for job_name in job_names:
        job = server[job_name]
        for build_id in job.get_build_ids():
            build = job.get_build(build_id)
            cutoff = date.today() - timedelta(days=report_days)
            if build.get_timestamp().date() < cutoff:
                break
            if build.get_status():
                # Running build have not status so don't add them
                jobs.append(build)
    return jobs


def last_build_worked(builds):
    return builds[0].is_good()


def write_html(html, html_file, mode='a'):
    with open(html_file, mode) as the_file:
        the_file.write(html + '\n')


def render_html_tmpl(template, target, templatevars=None):
    env = Environment(loader=FileSystemLoader(TEMPLATES))
    template = env.get_template(template)
    with open(target, 'w') as the_file:
        the_file.write(template.render(templatevars))


def write_spec_status_file(spec, urelease, builds, extract_build_info):
    spec_under = spec.replace('/', '_')
    spec_filename = '{}_{}.html'.format(spec_under, urelease)
    complete_runs = []
    for build in builds:
        if build.is_good():
            colour = GREEN
        else:
            colour = RED
        build_info = extract_build_info(build)
        complete_runs.append({
            'url': build_info['url'],
            'colour': colour,
            'buildtime': str(build_info['buildtime']),
        })
        render_html_tmpl(
            'job-details.tmpl', spec_filename,
            {'complete_runs': complete_runs})
    return spec_filename


def render_html(spec_dict, template, get_cell_data, report_days, status_file):
    ctxt = {}
    ctxt['reportruntime'] = strftime("%Y-%m-%d %H:%M:%S", gmtime())
    ctxt['reportdays'] = report_days
    ctxt['releasecombos'] = FULL_COMBOS
    ctxt['green'] = GREEN
    ctxt['red'] = RED
    ctxt['amber'] = AMBER
    ctxt['grey'] = GREY
    ctxt['celldata'] = get_cell_data(spec_dict)
    render_html_tmpl(template, status_file, ctxt)
